@@ -6,23 +6,27 @@
 //! types, and documentation. It supports multiple signatures for union types
 //! and overloads.
 
-use crate::{Db, docstring::get_parameter_documentation, find_node::covering_node};
+use crate::docstring::Docstring;
+use crate::goto::DefinitionsOrTargets;
+use crate::{Db, find_node::covering_node};
 use ruff_db::files::File;
 use ruff_db::parsed::parsed_module;
 use ruff_python_ast::{self as ast, AnyNodeRef};
+use ruff_python_parser::TokenKind;
 use ruff_text_size::{Ranged, TextRange, TextSize};
+use ty_python_semantic::ResolvedDefinition;
+use ty_python_semantic::SemanticModel;
 use ty_python_semantic::semantic_index::definition::Definition;
-use ty_python_semantic::types::{CallSignatureDetails, call_signature_details};
+use ty_python_semantic::types::{
+    CallSignatureDetails, call_signature_details, find_active_signature_from_details,
+};
 
-// Limitations of the current implementation:
-
-// TODO - If the target function is declared in a stub file but defined (implemented)
-// in a source file, the documentation will not reflect the a docstring that appears
-// only in the implementation. To do this, we'll need to map the function or
-// method in the stub to the implementation and extract the docstring from there.
+// TODO: We may want to add special-case handling for calls to constructors
+// so the class docstring is used in place of (or inaddition to) any docstring
+// associated with the __new__ or __init__ call.
 
 /// Information about a function parameter
-#[derive(Debug, Clone)]
+#[derive(Debug, Clone, PartialEq, Eq)]
 pub struct ParameterDetails {
     /// The parameter name (e.g., "param1")
     pub name: String,
@@ -34,12 +38,12 @@ pub struct ParameterDetails {
 }
 
 /// Information about a function signature
-#[derive(Debug, Clone)]
+#[derive(Debug, Clone, PartialEq, Eq)]
 pub struct SignatureDetails {
     /// Text representation of the full signature (including input parameters and return type).
     pub label: String,
     /// Documentation for the signature, typically from the function's docstring.
-    pub documentation: Option<String>,
+    pub documentation: Option<Docstring>,
     /// Information about each of the parameters in left-to-right order.
     pub parameters: Vec<ParameterDetails>,
     /// Index of the parameter that corresponds to the argument where the
@@ -48,7 +52,7 @@ pub struct SignatureDetails {
 }
 
 /// Signature help information for function calls
-#[derive(Debug, Clone)]
+#[derive(Debug, Clone, PartialEq, Eq)]
 pub struct SignatureHelpInfo {
     /// Information about each of the signatures for the function call. We
     /// need to handle multiple because of unions, overloads, and composite
@@ -66,9 +70,11 @@ pub fn signature_help(db: &dyn Db, file: File, offset: TextSize) -> Option<Signa
     // Get the call expression at the given position.
     let (call_expr, current_arg_index) = get_call_expr(&parsed, offset)?;
 
+    let model = SemanticModel::new(db, file);
+
     // Get signature details from the semantic analyzer.
     let signature_details: Vec<CallSignatureDetails<'_>> =
-        call_signature_details(db, file, call_expr);
+        call_signature_details(db, &model, call_expr);
 
     if signature_details.is_empty() {
         return None;
@@ -97,16 +103,44 @@ fn get_call_expr(
     parsed: &ruff_db::parsed::ParsedModuleRef,
     offset: TextSize,
 ) -> Option<(&ast::ExprCall, usize)> {
-    // Create a range from the offset for the covering_node function.
-    let range = TextRange::new(offset, offset);
+    let root_node: AnyNodeRef = parsed.syntax().into();
+
+    // Find the token under the cursor and use its offset to find the node
+    let token = parsed
+        .tokens()
+        .at_offset(offset)
+        .max_by_key(|token| match token.kind() {
+            TokenKind::Name
+            | TokenKind::String
+            | TokenKind::Complex
+            | TokenKind::Float
+            | TokenKind::Int => 1,
+            _ => 0,
+        })?;
 
     // Find the covering node at the given position that is a function call.
-    let covering_node = covering_node(parsed.syntax().into(), range)
-        .find_first(|node| matches!(node, AnyNodeRef::ExprCall(_)))
+    let call = covering_node(root_node, token.range())
+        .find_first(|node| {
+            if !node.is_expr_call() {
+                return false;
+            }
+
+            // Close the signature help if the cursor is at the closing parenthesis
+            if token.kind() == TokenKind::Rpar && node.end() == token.end() && offset == token.end()
+            {
+                return false;
+            }
+
+            if token.range().is_empty() && node.end() == token.end() {
+                return false;
+            }
+
+            true
+        })
         .ok()?;
 
     // Get the function call expression.
-    let AnyNodeRef::ExprCall(call_expr) = covering_node.node() else {
+    let AnyNodeRef::ExprCall(call_expr) = call.node() else {
         return None;
     };
 
@@ -149,7 +183,7 @@ fn create_signature_details_from_call_signature_details(
             details
                 .argument_to_parameter_mapping
                 .get(current_arg_index)
-                .and_then(|&param_index| param_index)
+                .and_then(|mapping| mapping.parameters.first().copied())
                 .or({
                     // If we can't find a mapping for this argument, but we have a current
                     // argument index, use that as the active parameter if it's within bounds.
@@ -161,46 +195,39 @@ fn create_signature_details_from_call_signature_details(
                 })
         };
 
+    let parameters = create_parameters_from_offsets(
+        &details.parameter_label_offsets,
+        &signature_label,
+        documentation.as_ref(),
+        &details.parameter_names,
+    );
     SignatureDetails {
-        label: signature_label.clone(),
-        documentation: Some(documentation),
-        parameters: create_parameters_from_offsets(
-            &details.parameter_label_offsets,
-            &signature_label,
-            db,
-            details.definition,
-            &details.parameter_names,
-        ),
+        label: signature_label,
+        documentation,
+        parameters,
         active_parameter,
     }
 }
 
 /// Determine appropriate documentation for a callable type based on its original type.
-fn get_callable_documentation(db: &dyn crate::Db, definition: Option<Definition>) -> String {
-    // TODO: If the definition is located within a stub file and no docstring
-    // is present, try to map the symbol to an implementation file and extract
-    // the docstring from that location.
-    if let Some(definition) = definition {
-        definition.docstring(db).unwrap_or_default()
-    } else {
-        String::new()
-    }
+fn get_callable_documentation(
+    db: &dyn crate::Db,
+    definition: Option<Definition>,
+) -> Option<Docstring> {
+    DefinitionsOrTargets::Definitions(vec![ResolvedDefinition::Definition(definition?)])
+        .docstring(db)
 }
 
 /// Create `ParameterDetails` objects from parameter label offsets.
 fn create_parameters_from_offsets(
     parameter_offsets: &[TextRange],
     signature_label: &str,
-    db: &dyn crate::Db,
-    definition: Option<Definition>,
+    docstring: Option<&Docstring>,
     parameter_names: &[String],
 ) -> Vec<ParameterDetails> {
     // Extract parameter documentation from the function's docstring if available.
-    let param_docs = if let Some(definition) = definition {
-        let docstring = definition.docstring(db);
-        docstring
-            .map(|doc| get_parameter_documentation(&doc))
-            .unwrap_or_default()
+    let param_docs = if let Some(docstring) = docstring {
+        docstring.parameter_documentation()
     } else {
         std::collections::HashMap::new()
     };
@@ -229,47 +256,9 @@ fn create_parameters_from_offsets(
         .collect()
 }
 
-/// Find the active signature index from `CallSignatureDetails`.
-/// The active signature is the first signature where all arguments present in the call
-/// have valid mappings to parameters (i.e., none of the mappings are None).
-fn find_active_signature_from_details(signature_details: &[CallSignatureDetails]) -> Option<usize> {
-    let first = signature_details.first()?;
-
-    // If there are no arguments in the mapping, just return the first signature.
-    if first.argument_to_parameter_mapping.is_empty() {
-        return Some(0);
-    }
-
-    // First, try to find a signature where all arguments have valid parameter mappings.
-    let perfect_match = signature_details.iter().position(|details| {
-        // Check if all arguments have valid parameter mappings (i.e., are not None).
-        details
-            .argument_to_parameter_mapping
-            .iter()
-            .all(Option::is_some)
-    });
-
-    if let Some(index) = perfect_match {
-        return Some(index);
-    }
-
-    // If no perfect match, find the signature with the most valid argument mappings.
-    let (best_index, _) = signature_details
-        .iter()
-        .enumerate()
-        .max_by_key(|(_, details)| {
-            details
-                .argument_to_parameter_mapping
-                .iter()
-                .filter(|mapping| mapping.is_some())
-                .count()
-        })?;
-
-    Some(best_index)
-}
-
 #[cfg(test)]
 mod tests {
+    use crate::docstring::Docstring;
     use crate::signature_help::SignatureHelpInfo;
     use crate::tests::{CursorTest, cursor_test};
 
@@ -279,11 +268,11 @@ mod tests {
             r#"
         def example_function(param1: str, param2: int) -> str:
             """This is a docstring for the example function.
-            
+
             Args:
                 param1: The first parameter as a string
                 param2: The second parameter as an integer
-            
+
             Returns:
                 A formatted string combining both parameters
             """
@@ -303,17 +292,19 @@ mod tests {
         // Verify that the docstring is extracted and included in the documentation
         let expected_docstring = concat!(
             "This is a docstring for the example function.\n",
-            "            \n",
-            "            Args:\n",
-            "                param1: The first parameter as a string\n",
-            "                param2: The second parameter as an integer\n",
-            "            \n",
-            "            Returns:\n",
-            "                A formatted string combining both parameters\n",
-            "            "
+            "\n",
+            "Args:\n",
+            "    param1: The first parameter as a string\n",
+            "    param2: The second parameter as an integer\n",
+            "\n",
+            "Returns:\n",
+            "    A formatted string combining both parameters\n",
         );
         assert_eq!(
-            signature.documentation,
+            signature
+                .documentation
+                .as_ref()
+                .map(Docstring::render_plaintext),
             Some(expected_docstring.to_string())
         );
 
@@ -523,9 +514,12 @@ mod tests {
         assert_eq!(param_y.documentation, Some("The y-coordinate".to_string()));
 
         // Should have the __init__ method docstring as documentation (not the class docstring)
-        let expected_docstring = "Initialize a point with x and y coordinates.\n                \n                Args:\n                    x: The x-coordinate\n                    y: The y-coordinate\n                ";
+        let expected_docstring = "Initialize a point with x and y coordinates.\n\nArgs:\n    x: The x-coordinate\n    y: The y-coordinate\n";
         assert_eq!(
-            signature.documentation,
+            signature
+                .documentation
+                .as_ref()
+                .map(Docstring::render_plaintext),
             Some(expected_docstring.to_string())
         );
     }
@@ -571,7 +565,13 @@ mod tests {
         let signature = &result.signatures[0];
 
         // Should have empty documentation for now
-        assert_eq!(signature.documentation, Some(String::new()));
+        assert_eq!(
+            signature
+                .documentation
+                .as_ref()
+                .map(Docstring::render_plaintext),
+            None
+        );
     }
 
     #[test]
@@ -648,7 +648,7 @@ mod tests {
             r#"
         def documented_function(param1: str, param2: int) -> str:
             """This is a function with parameter documentation.
-            
+
             Args:
                 param1: The first parameter description
                 param2: The second parameter description
@@ -679,6 +679,149 @@ mod tests {
         );
     }
 
+    #[test]
+    fn signature_help_after_closing_paren() {
+        let test = cursor_test(
+            r#"
+        def func1(v: str) -> str:
+            return v
+
+        r = func1("")<CURSOR>
+        print(r)
+        "#,
+        );
+
+        let result = test.signature_help();
+        assert!(
+            result.is_none(),
+            "Signature help should return None after closing paren"
+        );
+    }
+
+    #[test]
+    fn signature_help_after_nested_closing_paren() {
+        let test = cursor_test(
+            r#"
+        def inner_func(x: str) -> str:
+            return x.upper()
+
+        def outer_func(a: int, b: str) -> str:
+            return f"{a}: {b}"
+
+        result = outer_func(42, inner_func("hello")<CURSOR>
+        "#,
+        );
+
+        // Should return signature help for the outer function call
+        // even though cursor is after the closing paren of the inner call
+        let result = test
+            .signature_help()
+            .expect("Should have signature help for outer function");
+        assert_eq!(result.signatures.len(), 1);
+
+        let signature = &result.signatures[0];
+        assert!(signature.label.contains("a: int") && signature.label.contains("b: str"));
+
+        // Should be on the second parameter (b: str) since we're after the inner call
+        assert_eq!(signature.active_parameter, Some(1));
+        assert_eq!(result.active_signature, Some(0));
+    }
+
+    #[test]
+    fn signature_help_after_closing_paren_at_end_of_file() {
+        let test = cursor_test(
+            r#"
+            def test(a: int) -> int:
+                return 10
+
+            test("test")<CURSOR>"#,
+        );
+
+        // Should not return a signature help
+        assert_eq!(test.signature_help(), None);
+    }
+
+    #[test]
+    fn signature_help_after_closing_paren_in_expression() {
+        let test = cursor_test(
+            r#"
+            def test(a: int) -> int:
+                return 10
+
+            test("test")<CURSOR> + 10
+        "#,
+        );
+
+        // Should not return a signature help
+        assert_eq!(test.signature_help(), None);
+    }
+
+    #[test]
+    fn signature_help_after_closing_paren_nested() {
+        let test = cursor_test(
+            r#"
+        def inner(a: int) -> int:
+            return 10
+
+        def outer(a: int) -> None: ...
+
+        outer(inner("test")<CURSOR> + 10)
+        "#,
+        );
+
+        // Should return the outer signature help
+        let help = test.signature_help().expect("Should have outer help");
+
+        assert_eq!(help.signatures.len(), 1);
+
+        let signature = &help.signatures[0];
+        assert_eq!(signature.label, "(a: int) -> None");
+    }
+
+    #[test]
+    fn signature_help_stub_to_implementation_mapping() {
+        // Test that when a function is called from a stub file with no docstring,
+        // the signature help includes the docstring from the corresponding implementation file
+        let test = CursorTest::builder()
+            .source(
+                "main.py",
+                r#"
+                from lib import func
+                result = func(<CURSOR>
+"#,
+            )
+            .source(
+                "lib.pyi",
+                r#"
+                def func() -> str: ...
+"#,
+            )
+            .source(
+                "lib.py",
+                r#"
+                def func() -> str:
+                    """This function does something."""
+                    return ""
+"#,
+            )
+            .build();
+
+        let result = test.signature_help().expect("Should have signature help");
+        assert_eq!(result.signatures.len(), 1);
+
+        let signature = &result.signatures[0];
+        assert_eq!(signature.label, "() -> str");
+
+        let expected_docstring = "This function does something.\n";
+        assert_eq!(
+            signature
+                .documentation
+                .as_ref()
+                .map(Docstring::render_plaintext),
+            Some(expected_docstring.to_string())
+        );
+    }
+
     impl CursorTest {
         fn signature_help(&self) -> Option<SignatureHelpInfo> {
             crate::signature_help::signature_help(&self.db, self.cursor.file, self.cursor.offset)
